@@ -1,7 +1,7 @@
 /* List a tar archive, with support routines for reading a tar archive.
 
    Copyright (C) 1988, 1992, 1993, 1994, 1996, 1997, 1998, 1999, 2000,
-   2001, 2003 Free Software Foundation, Inc.
+   2001, 2003, 2004 Free Software Foundation, Inc.
 
    Written by John Gilmore, on 1985-08-26.
 
@@ -78,7 +78,7 @@ read_and (void (*do_something) (void))
       prev_status = status;
       tar_stat_destroy (&current_stat_info);
       xheader_destroy (&extended_header);
-      
+
       status = read_header (false);
       switch (status)
 	{
@@ -92,12 +92,17 @@ read_and (void (*do_something) (void))
 	     Ensure incoming names are null terminated.  */
 
 	  if (! name_match (current_stat_info.file_name)
-	      || (newer_mtime_option != TYPE_MINIMUM (time_t)
+	      || (NEWER_OPTION_INITIALIZED (newer_mtime_option)
 		  /* FIXME: We get mtime now, and again later; this causes
 		     duplicate diagnostics if header.mtime is bogus.  */
 		  && ((current_stat_info.stat.st_mtime
-		       = TIME_FROM_HEADER (current_header->header.mtime))
-		      < newer_mtime_option))
+		       = TIME_FROM_HEADER (current_header->header.mtime)),
+#ifdef ST_MTIM_NSEC
+		      /* FIXME: Grab fractional time stamps from
+			 extended header.  */
+		      current_stat_info.stat.st_mtim.ST_MTIM_NSEC = 0,
+#endif
+		      OLDER_STAT_TIME (current_stat_info.stat, m)))
 	      || excluded_name (current_stat_info.file_name))
 	    {
 	      switch (current_header->header.typeflag)
@@ -113,6 +118,8 @@ read_and (void (*do_something) (void))
 			   quotearg_colon (current_stat_info.file_name)));
 		  /* Fall through.  */
 		default:
+		  decode_header (current_header,
+				 &current_stat_info, &current_format, 0);
 		  skip_member ();
 		  continue;
 		}
@@ -140,6 +147,7 @@ read_and (void (*do_something) (void))
 		break;
 	      WARN ((0, 0, _("A lone zero block at %s"),
 		     STRINGIFY_BIGINT (current_block_ordinal (), buf)));
+	      break;
 	    }
 	  status = prev_status;
 	  continue;
@@ -165,6 +173,15 @@ read_and (void (*do_something) (void))
 
 	    case HEADER_ZERO_BLOCK:
 	    case HEADER_SUCCESS:
+	      if (block_number_option)
+		{
+		  char buf[UINTMAX_STRSIZE_BOUND];
+		  off_t block_ordinal = current_block_ordinal ();
+		  block_ordinal -= recent_long_name_blocks;
+		  block_ordinal -= recent_long_link_blocks;
+		  fprintf (stdlis, _("block %s: "),
+			   STRINGIFY_BIGINT (block_ordinal, buf));
+		}
 	      ERROR ((0, 0, _("Skipping to next header")));
 	      break;
 
@@ -355,7 +372,7 @@ read_header (bool raw_extended_headers)
 		xalloc_die ();
 
 	      header_copy = xmalloc (size + 1);
-	      
+
 	      if (header->header.typeflag == GNUTYPE_LONGNAME)
 		{
 		  if (next_long_name)
@@ -370,7 +387,7 @@ read_header (bool raw_extended_headers)
 		  next_long_link = header_copy;
 		  next_long_link_blocks = size / BLOCKSIZE;
 		}
-	      
+
 	      set_next_block_after (header);
 	      *header_copy = *header;
 	      bp = header_copy->buffer + BLOCKSIZE;
@@ -386,7 +403,7 @@ read_header (bool raw_extended_headers)
 		  written = available_space_after (data_block);
 		  if (written > size)
 		    written = size;
-		  
+
 		  memcpy (bp, data_block->buffer, written);
 		  bp += written;
 		  set_next_block_after ((union block *)
@@ -402,7 +419,7 @@ read_header (bool raw_extended_headers)
 	      xheader_read (header, OFF_FROM_HEADER (header->header.size));
 	      xheader_decode_global ();
 	    }
-      
+
 	  /* Loop!  */
 
 	}
@@ -519,7 +536,7 @@ decode_header (union block *header, struct tar_stat_info *stat_info,
   assign_string (&stat_info->gname, header->header.gname);
   stat_info->devmajor = MAJOR_FROM_HEADER (header->header.devmajor);
   stat_info->devminor = MINOR_FROM_HEADER (header->header.devminor);
-  
+
   stat_info->stat.st_atime = start_time;
   stat_info->stat.st_ctime = start_time;
 
@@ -558,12 +575,13 @@ decode_header (union block *header, struct tar_stat_info *stat_info,
 	      || !gname_to_gid (header->header.gname, &stat_info->stat.st_gid))
 	    stat_info->stat.st_gid = GID_FROM_HEADER (header->header.gid);
 	}
-      
+
       switch (header->header.typeflag)
 	{
 	case BLKTYPE:
 	case CHRTYPE:
-	  stat_info->stat.st_rdev = makedev (stat_info->devmajor, stat_info->devminor);
+	  stat_info->stat.st_rdev = makedev (stat_info->devmajor,
+					     stat_info->devminor);
 	  break;
 
 	default:
@@ -571,8 +589,16 @@ decode_header (union block *header, struct tar_stat_info *stat_info,
 	}
     }
 
-  current_stat_info.archive_file_size = current_stat_info.stat.st_size;
+  stat_info->archive_file_size = stat_info->stat.st_size;
   xheader_decode (stat_info);
+
+  if (sparse_member_p (stat_info))
+    {
+      sparse_fixup_header (stat_info);
+      stat_info->is_sparse = true;
+    }
+  else
+    stat_info->is_sparse = false;
 }
 
 /* Convert buffer at WHERE0 of size DIGS from external format to
@@ -899,7 +925,7 @@ tartime (time_t t)
 #else
   /* Use ISO 8610 format.  See:
      http://www.cl.cam.ac.uk/~mgk25/iso-time.html  */
-  struct tm *tm = localtime (&t);
+  struct tm *tm = utc_option ? gmtime (&t) : localtime (&t);
   if (tm)
     {
       sprintf (buffer, "%04ld-%02d-%02d %02d:%02d:%02d",
@@ -953,7 +979,7 @@ print_header (struct tar_stat_info *st, off_t block_ordinal)
   char modes[11];
   char const *time_stamp;
   char *temp_name = st->orig_file_name ? st->orig_file_name : st->file_name;
-  
+
   /* These hold formatted ints.  */
   char uform[UINTMAX_STRSIZE_BOUND], gform[UINTMAX_STRSIZE_BOUND];
   char *user, *group;
@@ -1100,16 +1126,10 @@ print_header (struct tar_stat_info *st, off_t block_ordinal)
 	  strcat (size,
 		  STRINGIFY_BIGINT (minor (st->stat.st_rdev), uintbuf));
 	  break;
-	case GNUTYPE_SPARSE:
-	  strcpy (size,
-		  STRINGIFY_BIGINT
-		  (UINTMAX_FROM_HEADER (current_header
-					->oldgnu_header.realsize),
-		   uintbuf));
-	  break;
+
 	default:
 	  /* st->stat.st_size keeps stored file size */
-	  strcpy (size, STRINGIFY_BIGINT (st->archive_file_size, uintbuf));
+	  strcpy (size, STRINGIFY_BIGINT (st->stat.st_size, uintbuf));
 	  break;
 	}
 
@@ -1186,7 +1206,7 @@ print_header (struct tar_stat_info *st, off_t block_ordinal)
 
 /* Print a similar line when we make a directory automatically.  */
 void
-print_for_mkdir (char *pathname, int length, mode_t mode)
+print_for_mkdir (char *dirname, int length, mode_t mode)
 {
   char modes[11];
 
@@ -1205,7 +1225,7 @@ print_for_mkdir (char *pathname, int length, mode_t mode)
 	}
 
       fprintf (stdlis, "%s %*s %.*s\n", modes, ugswidth + DATEWIDTH,
-	       _("Creating directory:"), length, quotearg (pathname));
+	       _("Creating directory:"), length, quotearg (dirname));
     }
 }
 
@@ -1221,6 +1241,19 @@ skip_file (off_t size)
       save_sizeleft = size;
     }
 
+  if (seekable_archive)
+    {
+      off_t nblk = seek_archive (size);
+      if (nblk >= 0)
+	{
+	  size -= nblk * BLOCKSIZE;
+	  if (multi_volume_option) /* Argh.. */
+	    save_sizeleft -= nblk * BLOCKSIZE;
+	}
+      else
+	seekable_archive = false;
+    }
+  
   while (size > 0)
     {
       x = find_next_block ();
@@ -1234,27 +1267,18 @@ skip_file (off_t size)
     }
 }
 
-/* Skip the current member in the archive.  */
+/* Skip the current member in the archive.
+   NOTE: Current header must be decoded before calling this function. */
 void
 skip_member (void)
 {
   char save_typeflag = current_header->header.typeflag;
   set_next_block_after (current_header);
+  
+  assign_string (&save_name, current_stat_info.file_name);
 
-  if (current_format == OLDGNU_FORMAT
-      && current_header->oldgnu_header.isextended)
-    {
-      union block *exhdr;
-      do
-	{
-	  exhdr = find_next_block ();
-	  if (!exhdr)
-	    FATAL_ERROR ((0, 0, _("Unexpected EOF in archive")));
-	  set_next_block_after (exhdr);
-	}
-      while (exhdr->sparse_header.isextended);
-    }
-
-  if (save_typeflag != DIRTYPE)
+  if (current_stat_info.is_sparse)
+    sparse_skip_file (&current_stat_info);
+  else if (save_typeflag != DIRTYPE)
     skip_file (current_stat_info.stat.st_size);
 }
