@@ -1,6 +1,4 @@
 #! /bin/sh
-# Deleting members with long file names.
-
 # Copyright (C) 2001 Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,7 +17,8 @@
 # 02111-1307, USA.
 
 . ./preset
-. $srcdir/before
+TAR_ARCHIVE_FORMATS="gnu oldgnu posix"
+. $srcdir/before 
 
 set -e
 prefix=This_is_a_very_long_file_name_prefix_that_is_designed_to_cause_problems_with_file_names_that_run_into_a_limit_of_the_posix_tar_formatXX
@@ -28,21 +27,18 @@ for i in 1 2 3 4 5 6 7 8 9
 do touch $prefix$i
 done
 tar -cf archive ./$prefix*
-tar --delete -f archive ${prefix}5
+tar --delete -f archive ./${prefix}5
 tar -tf archive
 
 out="\
-${prefix}1
-${prefix}2
-${prefix}3
-${prefix}4
-${prefix}6
-${prefix}7
-${prefix}8
-${prefix}9
-"
-
-err="tar: Removing leading \`./' from member names
+./${prefix}1
+./${prefix}2
+./${prefix}3
+./${prefix}4
+./${prefix}6
+./${prefix}7
+./${prefix}8
+./${prefix}9
 "
 
 . $srcdir/after
